#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 3
#  of the License, or (at your option) any later version.

#  Author::    Dieter Spaeth  (mailto:dieter.spaeth@gmx.de)
#  Copyright:: Copyright (c) 2010 Dieter Spaeth

module RailsJqueryBundle
  module JqueryJsHelper
    include RailsJqueryBundle::Constants::Jquery
    include RailsJqueryBundle::Constants::Jquery::Ui
    
    # Creates a includetag for only for jQuery:
    # for production environment:
    # * javascripts/jquery/JQUERY_VERSION/minified/jquery.js
    # all other environments:
    # * javascripts/jquery/JQUERY_VERSION/jquery.js
    #
    # @return [String] Javascript includetag for jquery
    # @example include jQuery
    #   javascript_include_tag
    def jquery_js_tag
      javascript_include_tag([JQUERY_BASE,JQUERY_VERSION,production_folder, jquery_filename].compact.join('/')).html_safe
    end

    # Creates a includetag(s) for jQuery and jQuery-Ui:
    # for production environment:
    # * javascripts/jquery/ui/JQUERY_UI_VERSION/minified/*js
    # all other environments:
    # * javascripts/jquery/ui/JQUERY_UI_VERSION/*js
    # jquery.ui.core.js is included in evry case
    # 
    # @param [Hash, Symbol] wanted_parts
    # @option wanted_parts [Symbol] :all  includes jquery-ui.all.js, jquery-ui-i18n.js
    # @option wanted_parts [Array] :ui Ui-Files which should all be included
    # @option wanted_parts [Array] :effects Effect-Files which should all be included
    # @option wanted_parts [Array] :i18n i18n-Files which should be included
    # 
    # @return [String] Javascript includetag for jquery-ui and i18n
    # @example include some effects, ui and i18n
    #    jquery_ui_js_tag :ui => [:datepicker], :effects => [:shake] , :i18n => "datepicker-de"
    # @example include all effects, all ui and  all i18n
    #    jquery_ui_js_tag :all
    def jquery_ui_js_tag(wanted_parts=:all)
      [jquery_js_tag,
       jquery_ui_js_tags(wanted_parts)
      ].flatten.
        join("\n").
        html_safe
    end

    private
    def jquery_filename
      if is_production?
        JQUERY_FILENAME_MIN
      else
        JQUERY_FILENAME
      end
    end

    def production_folder
      is_production? ? PRODUCTION_SUB_FOLDER : nil
    end

    def jquery_ui_js_tags(wanted_parts=:all)
      if (wanted_parts==:all)
        [create_ui_js_tag(JQUERY_UI_ALL_FILENAME),
          create_i18n_js_tag(JQUERY_UI_ALL_I18N_FILENAME)
        ]
      else
        wanted_parts[:ui] ||= []
        wanted_parts[:effects] ||= []
        wanted_parts[:i18n] ||= []

        # Move core to first element or include it, if it isn't
        # allready there
        wanted_parts[:ui].delete(:core)
        wanted_parts[:ui].unshift(:core)

        ui_parts = wanted_parts[:ui].map{|part|
          create_ui_js_tag(jquery_ui_ui_filename(part) )
        }

        effects_parts = wanted_parts[:effects].map{|part|
          create_ui_js_tag(jquery_ui_effects_filename(part))
        }

        i18n_parts = wanted_parts[:i18n].map{|part|
          create_i18n_js_tag(JQUERY_UI_I18N_FILE_PATTERN % part.to_s )
        }

        [ui_parts, effects_parts, i18n_parts].flatten
      end

    end

    def jquery_ui_ui_filename(filename_part)
      if is_production?
        JQUERY_UI_UI_FILE_PATTERN_MIN
      else
        JQUERY_UI_UI_FILE_PATTERN
      end % filename_part.to_s
    end

    def jquery_ui_effects_filename(filename_part)
      if is_production?
        JQUERY_UI_EFFECTS_FILE_PATTERN_MIN
      else
        JQUERY_UI_EFFECTS_FILE_PATTERN
      end % filename_part.to_s
    end

    def create_ui_js_tag(filename)
      javascript_include_tag [JQUERY_UI_BASE ,
        JQUERY_UI_VERSION,
        production_folder,
        filename].compact.join('/')
    end

    def create_i18n_js_tag(filename)
      javascript_include_tag [JQUERY_UI_BASE,
        JQUERY_UI_VERSION,
        'i18n',
        filename].compact.join('/')
    end

    def is_production?
      Rails.env.production?
    end

  end
end
